@@ -105,7 +105,7 @@ fn get_final_size(in_width: f64, in_height: f64, size_mode: &SizeMode) -> (f64,
     (out_width, out_height)
 }
 
-fn render_to_pixbuf_at_size(
+pub fn render_to_pixbuf_at_size(
     renderer: &CairoRenderer<'_>,
     document_width: f64,
     document_height: f64,
@@ -133,12 +133,7 @@ fn render_to_pixbuf_at_size(
             desired_height / document_height,
         );
 
-        let viewport = cairo::Rectangle {
-            x: 0.0,
-            y: 0.0,
-            width: document_width,
-            height: document_height,
-        };
+        let viewport = cairo::Rectangle::new(0.0, 0.0, document_width, document_height);
 
         // We do it with a cr transform so we can scale non-proportionally.
         renderer.render_document(&cr, &viewport)?;
