@@ -12,6 +12,7 @@ use std::cell::RefCell;
 use std::convert::TryFrom;
 use std::f64::consts::*;
 use std::rc::{Rc, Weak};
+use std::sync::Arc;
 
 use crate::accept_language::UserLanguage;
 use crate::aspect_ratio::AspectRatio;
@@ -36,7 +37,7 @@ use crate::properties::{
     ClipRule, ComputedValues, FillRule, Filter, Isolation, MaskType, MixBlendMode, Opacity,
     Overflow, PaintTarget, ShapeRendering, StrokeLinecap, StrokeLinejoin, TextRendering,
 };
-use crate::rect::{IRect, Rect};
+use crate::rect::{rect_to_transform, IRect, Rect};
 use crate::session::Session;
 use crate::surface_utils::{
     shared_surface::ExclusiveImageSurface, shared_surface::SharedImageSurface,
@@ -537,7 +538,7 @@ impl DrawingCtx {
         let node = clip_node.as_ref().unwrap();
         let units = borrow_element_as!(node, ClipPath).get_units();
 
-        if let Ok(transform) = bbox.rect_to_transform(units) {
+        if let Ok(transform) = rect_to_transform(&bbox.rect, units) {
             let cascaded = CascadedValues::new_from_node(node);
             let values = cascaded.get();
 
@@ -758,7 +759,7 @@ impl DrawingCtx {
                                 .create_similar_surface_for_toplevel_viewport(&self.cr.target())?,
                         )?,
                         Filter::List(_) => {
-                            cairo::Context::new(&*self.create_surface_for_toplevel_viewport()?)?
+                            cairo::Context::new(self.create_surface_for_toplevel_viewport()?)?
                         }
                     };
 
@@ -807,7 +808,7 @@ impl DrawingCtx {
                                         None,
                                         self.session(),
                                     )
-                                    .to_user_space(&bbox, &params, values),
+                                    .to_user_space(&bbox.rect, &params, values),
                             );
 
                             let fill_paint_source = Rc::new(
@@ -822,7 +823,7 @@ impl DrawingCtx {
                                         None,
                                         self.session(),
                                     )
-                                    .to_user_space(&bbox, &params, values),
+                                    .to_user_space(&bbox.rect, &params, values),
                             );
 
                             // Filter functions (like "blend()", not the <filter> element) require
@@ -969,7 +970,15 @@ impl DrawingCtx {
         current_color: RGBA,
         node_bbox: BoundingBox,
     ) -> Result<SharedImageSurface, RenderingError> {
-        let surface = if let Ok(specs) = filter_list
+        // We try to convert each item in the filter_list to a FilterSpec.
+        //
+        // However, the spec mentions, "If the filter references a non-existent object or
+        // the referenced object is not a filter element, then the whole filter chain is
+        // ignored." - https://www.w3.org/TR/filter-effects/#FilterProperty
+        //
+        // So, run through the filter_list and collect into a Result<Vec<FilterSpec>>.
+        // This will return an Err if any of the conversions failed.
+        let filter_specs = filter_list
             .iter()
             .map(|filter_value| {
                 filter_value.to_filter_spec(
@@ -980,25 +989,37 @@ impl DrawingCtx {
                     node_name,
                 )
             })
-            .collect::<Result<Vec<FilterSpec>, _>>()
-        {
-            specs.iter().try_fold(surface_to_filter, |surface, spec| {
-                filters::render(
-                    spec,
-                    stroke_paint_source.clone(),
-                    fill_paint_source.clone(),
-                    surface,
-                    acquired_nodes,
-                    self,
-                    self.get_transform(),
-                    node_bbox,
-                )
-            })?
-        } else {
-            surface_to_filter
-        };
+            .collect::<Result<Vec<FilterSpec>, _>>();
 
-        Ok(surface)
+        match filter_specs {
+            Ok(specs) => {
+                // Start with the surface_to_filter, and apply each filter spec in turn;
+                // the final result is our return value.
+                specs.iter().try_fold(surface_to_filter, |surface, spec| {
+                    filters::render(
+                        spec,
+                        stroke_paint_source.clone(),
+                        fill_paint_source.clone(),
+                        surface,
+                        acquired_nodes,
+                        self,
+                        self.get_transform(),
+                        node_bbox,
+                    )
+                })
+            }
+
+            Err(e) => {
+                rsvg_log!(
+                    self.session,
+                    "not rendering filter list on node {} because it was in error: {}",
+                    node_name,
+                    e
+                );
+                // just return the original surface without filtering it
+                Ok(surface_to_filter)
+            }
+        }
     }
 
     fn set_gradient(&mut self, gradient: &UserSpaceGradient) -> Result<(), cairo::Error> {
@@ -1236,16 +1257,29 @@ impl DrawingCtx {
         Ok(())
     }
 
+    pub fn compute_path_extents(&self, path: &Path) -> Result<Option<Rect>, RenderingError> {
+        if path.is_empty() {
+            return Ok(None);
+        }
+
+        let surface = cairo::RecordingSurface::create(cairo::Content::ColorAlpha, None)?;
+        let cr = cairo::Context::new(&surface)?;
+
+        path.to_cairo(&cr, false)?;
+        let (x0, y0, x1, y1) = cr.path_extents()?;
+
+        Ok(Some(Rect::new(x0, y0, x1, y1)))
+    }
+
     pub fn draw_shape(
         &mut self,
-        view_params: &ViewParams,
         shape: &Shape,
         stacking_ctx: &StackingContext,
         acquired_nodes: &mut AcquiredNodes<'_>,
         values: &ComputedValues,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        if shape.path.is_empty() {
+        if shape.extents.is_none() {
             return Ok(self.empty_bbox());
         }
 
@@ -1275,10 +1309,12 @@ impl DrawingCtx {
                 cr.set_fill_rule(cairo::FillRule::from(shape.fill_rule));
 
                 path_helper.set()?;
-                let bbox = compute_stroke_and_fill_box(&cr, &shape.stroke, &shape.stroke_paint)?;
-
-                let stroke_paint = shape.stroke_paint.to_user_space(&bbox, view_params, values);
-                let fill_paint = shape.fill_paint.to_user_space(&bbox, view_params, values);
+                let bbox = compute_stroke_and_fill_box(
+                    &cr,
+                    &shape.stroke,
+                    &shape.stroke_paint,
+                    &dc.initial_viewport,
+                )?;
 
                 if shape.is_visible {
                     for &target in &shape.paint_order.targets {
@@ -1287,12 +1323,22 @@ impl DrawingCtx {
                         match target {
                             PaintTarget::Fill => {
                                 path_helper.set()?;
-                                dc.fill(&cr, an, &fill_paint)?;
+                                dc.fill(&cr, an, &shape.fill_paint)?;
                             }
 
                             PaintTarget::Stroke => {
                                 path_helper.set()?;
-                                dc.stroke(&cr, an, &stroke_paint)?;
+                                let backup_matrix = if shape.stroke.non_scaling {
+                                    let matrix = cr.matrix();
+                                    cr.set_matrix(dc.initial_viewport.transform.into());
+                                    Some(matrix)
+                                } else {
+                                    None
+                                };
+                                dc.stroke(&cr, an, &shape.stroke_paint)?;
+                                if let Some(matrix) = backup_matrix {
+                                    cr.set_matrix(matrix);
+                                }
                             }
 
                             PaintTarget::Markers => {
@@ -1420,8 +1466,12 @@ impl DrawingCtx {
             }
 
             path.to_cairo(&self.cr, false)?;
-            let bbox =
-                compute_stroke_and_fill_box(&self.cr, &span.stroke, &span.stroke_paint_source)?;
+            let bbox = compute_stroke_and_fill_box(
+                &self.cr,
+                &span.stroke,
+                &span.stroke_paint,
+                &self.initial_viewport,
+            )?;
             self.cr.new_path();
 
             if span.is_visible {
@@ -1612,8 +1662,8 @@ impl DrawingCtx {
         use_rect: Rect,
         link: &NodeId,
         clipping: bool,
-        fill_paint: PaintSource,
-        stroke_paint: PaintSource,
+        fill_paint: Arc<PaintSource>,
+        stroke_paint: Arc<PaintSource>,
     ) -> Result<BoundingBox, RenderingError> {
         // <use> is an element that is used directly, unlike
         // <pattern>, which is used through a fill="url(#...)"
@@ -1678,16 +1728,27 @@ impl DrawingCtx {
 
         let use_element = node.borrow_element();
 
-        let res = if is_element_of_type!(child, Symbol) {
-            // if the <use> references a <symbol>, it gets handled specially
+        let defines_a_viewport = if is_element_of_type!(child, Symbol) {
+            let symbol = borrow_element_as!(child, Symbol);
+            Some((symbol.get_viewbox(), symbol.get_preserve_aspect_ratio()))
+        } else if is_element_of_type!(child, Svg) {
+            let svg = borrow_element_as!(child, Svg);
+            Some((svg.get_viewbox(), svg.get_preserve_aspect_ratio()))
+        } else {
+            None
+        };
+
+        let res = if let Some((viewbox, preserve_aspect_ratio)) = defines_a_viewport {
+            // <symbol> and <svg> define a viewport, as described in the specification:
+            // https://www.w3.org/TR/SVG2/struct.html#UseElement
+            // https://gitlab.gnome.org/GNOME/librsvg/-/issues/875#note_1482705
 
             let elt = child.borrow_element();
 
-            let symbol = borrow_element_as!(child, Symbol);
-            let symbol_values = elt.get_computed_values();
+            let values = elt.get_computed_values();
 
             // FIXME: do we need to look at preserveAspectRatio.slice, like in draw_image()?
-            let clip_mode = if !symbol_values.is_overflow() {
+            let clip_mode = if !values.is_overflow() {
                 ClipMode::ClipToViewport
             } else {
                 ClipMode::NoClip
@@ -1708,12 +1769,8 @@ impl DrawingCtx {
                 clipping,
                 None,
                 &mut |an, dc, _transform| {
-                    let _params = dc.push_new_viewport(
-                        symbol.get_viewbox(),
-                        use_rect,
-                        symbol.get_preserve_aspect_ratio(),
-                        clip_mode,
-                    );
+                    let _params =
+                        dc.push_new_viewport(viewbox, use_rect, preserve_aspect_ratio, clip_mode);
 
                     child.draw_children(
                         an,
@@ -1790,8 +1847,8 @@ impl DrawingCtx {
 
 /// Create a Pango context with a particular configuration.
 pub fn create_pango_context(font_options: &FontOptions, transform: &Transform) -> pango::Context {
-    let font_map = pangocairo::FontMap::default().unwrap();
-    let context = font_map.create_context().unwrap();
+    let font_map = pangocairo::FontMap::default();
+    let context = font_map.create_context();
 
     context.set_round_glyph_positions(false);
 
@@ -1960,7 +2017,8 @@ impl CompositingAffines {
 fn compute_stroke_and_fill_extents(
     cr: &cairo::Context,
     stroke: &Stroke,
-    stroke_paint_source: &PaintSource,
+    stroke_paint_source: &UserSpacePaintSource,
+    initial_viewport: &Viewport,
 ) -> Result<PathExtents, RenderingError> {
     // Dropping the precision of cairo's bezier subdivision, yielding 2x
     // _rendering_ time speedups, are these rather expensive operations
@@ -1991,9 +2049,19 @@ fn compute_stroke_and_fill_extents(
     // bounding box if so.
 
     let stroke_extents = if !stroke.width.approx_eq_cairo(0.0)
-        && !matches!(stroke_paint_source, PaintSource::None)
+        && !matches!(stroke_paint_source, UserSpacePaintSource::None)
     {
+        let backup_matrix = if stroke.non_scaling {
+            let matrix = cr.matrix();
+            cr.set_matrix(initial_viewport.transform.into());
+            Some(matrix)
+        } else {
+            None
+        };
         let (x0, y0, x1, y1) = cr.stroke_extents()?;
+        if let Some(matrix) = backup_matrix {
+            cr.set_matrix(matrix);
+        }
         Some(Rect::new(x0, y0, x1, y1))
     } else {
         None
@@ -2018,9 +2086,11 @@ fn compute_stroke_and_fill_extents(
 fn compute_stroke_and_fill_box(
     cr: &cairo::Context,
     stroke: &Stroke,
-    stroke_paint_source: &PaintSource,
+    stroke_paint_source: &UserSpacePaintSource,
+    initial_viewport: &Viewport,
 ) -> Result<BoundingBox, RenderingError> {
-    let extents = compute_stroke_and_fill_extents(cr, stroke, stroke_paint_source)?;
+    let extents =
+        compute_stroke_and_fill_extents(cr, stroke, stroke_paint_source, initial_viewport)?;
 
     let ink_rect = match (extents.fill, extents.stroke) {
         (None, None) => None,
@@ -2171,7 +2241,7 @@ impl From<TextRendering> for cairo::Antialias {
 impl From<cairo::Matrix> for Transform {
     #[inline]
     fn from(m: cairo::Matrix) -> Self {
-        Self::new_unchecked(m.xx, m.yx, m.xy, m.yy, m.x0, m.y0)
+        Self::new_unchecked(m.xx(), m.yx(), m.xy(), m.yy(), m.x0(), m.y0())
     }
 }
 
