@@ -6,13 +6,13 @@ use std::slice::Iter;
 
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::DrawingCtx;
-use crate::element::{Draw, Element, ElementResult, SetAttributes};
+use crate::drawing_ctx::{DrawingCtx, ViewParams};
+use crate::element::{set_attribute, Draw, Element, SetAttributes};
 use crate::error::ValueErrorKind;
 use crate::filter_func::FilterFunction;
-use crate::filters::{extract_filter_from_filter_node, FilterResolveError, FilterSpec};
+use crate::filters::{FilterResolveError, FilterSpec, UserSpacePrimitive};
 use crate::length::*;
-use crate::node::NodeBorrow;
+use crate::node::{Node, NodeBorrow};
 use crate::parsers::{Parse, ParseValue};
 use crate::rect::Rect;
 use crate::session::Session;
@@ -71,20 +71,26 @@ impl Filter {
 }
 
 impl SetAttributes for Filter {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "filterUnits") => self.filter_units = attr.parse(value)?,
-                expanded_name!("", "x") => self.x = attr.parse(value)?,
-                expanded_name!("", "y") => self.y = attr.parse(value)?,
-                expanded_name!("", "width") => self.width = attr.parse(value)?,
-                expanded_name!("", "height") => self.height = attr.parse(value)?,
-                expanded_name!("", "primitiveUnits") => self.primitive_units = attr.parse(value)?,
+                expanded_name!("", "filterUnits") => {
+                    set_attribute(&mut self.filter_units, attr.parse(value), session)
+                }
+                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
+                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
+                expanded_name!("", "width") => {
+                    set_attribute(&mut self.width, attr.parse(value), session)
+                }
+                expanded_name!("", "height") => {
+                    set_attribute(&mut self.height, attr.parse(value), session)
+                }
+                expanded_name!("", "primitiveUnits") => {
+                    set_attribute(&mut self.primitive_units, attr.parse(value), session)
+                }
                 _ => (),
             }
         }
-
-        Ok(())
     }
 }
 
@@ -114,12 +120,100 @@ impl FilterValue {
             ),
 
             FilterValue::Function(ref func) => {
-                func.to_filter_spec(user_space_params, current_color)
+                Ok(func.to_filter_spec(user_space_params, current_color))
             }
         }
     }
 }
 
+/// Holds the viewport parameters for both objectBoundingBox and userSpaceOnUse units.
+///
+/// When collecting a set of filter primitives (`feFoo`) into a [`FilterSpec`], which is
+/// in user space, we need to convert each primitive's units into user space units.  So,
+/// pre-compute both cases and pass them around.
+///
+/// This struct needs a better name; I didn't want to make it seem specific to filters by
+/// calling `FiltersViewParams` or `FilterCollectionProcessViewParams`.  Maybe the
+/// original [`ViewParams`] should be this struct, with both cases included...
+pub struct ViewParamsGen {
+    object_bounding_box: ViewParams,
+    user_space_on_use: ViewParams,
+}
+
+impl ViewParamsGen {
+    pub fn new(draw_ctx: &DrawingCtx) -> Self {
+        ViewParamsGen {
+            object_bounding_box: draw_ctx.get_view_params_for_units(CoordUnits::ObjectBoundingBox),
+            user_space_on_use: draw_ctx.get_view_params_for_units(CoordUnits::UserSpaceOnUse),
+        }
+    }
+
+    fn get(&self, units: CoordUnits) -> &ViewParams {
+        match units {
+            CoordUnits::ObjectBoundingBox => &self.object_bounding_box,
+            CoordUnits::UserSpaceOnUse => &self.user_space_on_use,
+        }
+    }
+}
+
+fn extract_filter_from_filter_node(
+    filter_node: &Node,
+    acquired_nodes: &mut AcquiredNodes<'_>,
+    session: &Session,
+    filter_view_params: &ViewParamsGen,
+) -> Result<FilterSpec, FilterResolveError> {
+    assert!(is_element_of_type!(filter_node, Filter));
+
+    let filter_element = filter_node.borrow_element();
+
+    let user_space_filter = {
+        let filter_values = filter_element.get_computed_values();
+
+        let filter = borrow_element_as!(filter_node, Filter);
+
+        filter.to_user_space(&NormalizeParams::new(
+            filter_values,
+            filter_view_params.get(filter.get_filter_units()),
+        ))
+    };
+
+    let primitive_view_params = filter_view_params.get(user_space_filter.primitive_units);
+
+    let primitives = filter_node
+        .children()
+        .filter(|c| c.is_element())
+        // Keep only filter primitives (those that implement the Filter trait)
+        .filter(|c| c.borrow_element().as_filter_effect().is_some())
+        .map(|primitive_node| {
+            let elt = primitive_node.borrow_element();
+            let effect = elt.as_filter_effect().unwrap();
+
+            let primitive_name = format!("{}", primitive_node);
+
+            let primitive_values = elt.get_computed_values();
+            let params = NormalizeParams::new(primitive_values, primitive_view_params);
+
+            effect
+                .resolve(acquired_nodes, &primitive_node)
+                .map_err(|e| {
+                    rsvg_log!(
+                        session,
+                        "(filter primitive {} returned an error: {})",
+                        primitive_name,
+                        e
+                    );
+                    e
+                })
+                .map(|primitive| primitive.into_user_space(&params))
+        })
+        .collect::<Result<Vec<UserSpacePrimitive>, FilterResolveError>>()?;
+
+    Ok(FilterSpec {
+        user_space_filter,
+        primitives,
+    })
+}
+
 fn filter_spec_from_filter_node(
     acquired_nodes: &mut AcquiredNodes<'_>,
     draw_ctx: &DrawingCtx,
@@ -128,6 +222,8 @@ fn filter_spec_from_filter_node(
 ) -> Result<FilterSpec, FilterResolveError> {
     let session = draw_ctx.session().clone();
 
+    let filter_view_params = ViewParamsGen::new(draw_ctx);
+
     acquired_nodes
         .acquire(node_id)
         .map_err(|e| {
@@ -145,19 +241,12 @@ fn filter_spec_from_filter_node(
             let element = node.borrow_element();
 
             match *element {
-                Element::Filter(_) => {
-                    if element.is_in_error() {
-                        rsvg_log!(
-                            session,
-                            "element {} will not be filtered since its filter \"{}\" is in error",
-                            node_being_filtered_name,
-                            node_id,
-                        );
-                        Err(FilterResolveError::ChildNodeInError)
-                    } else {
-                        extract_filter_from_filter_node(node, acquired_nodes, draw_ctx)
-                    }
-                }
+                Element::Filter(_) => extract_filter_from_filter_node(
+                    node,
+                    acquired_nodes,
+                    &session,
+                    &filter_view_params,
+                ),
 
                 _ => {
                     rsvg_log!(
