@@ -24,11 +24,11 @@ use crate::filters::{
     gaussian_blur::GaussianBlur,
     merge::{Merge, MergeNode},
     offset::Offset,
-    FilterResolveError, FilterSpec, Input, Primitive, PrimitiveParams, ResolvedPrimitive,
+    FilterSpec, Input, Primitive, PrimitiveParams, ResolvedPrimitive,
 };
 use crate::length::*;
 use crate::paint_server::resolve_color;
-use crate::parsers::{CustomIdent, NumberOrPercentage, Parse};
+use crate::parsers::{CustomIdent, NumberOptionalNumber, NumberOrPercentage, Parse};
 use crate::unit_interval::UnitInterval;
 
 /// CSS Filter functions from the Filter Effects Module Level 1
@@ -271,7 +271,7 @@ impl Blur {
         let gaussian_blur = ResolvedPrimitive {
             primitive: Primitive::default(),
             params: PrimitiveParams::GaussianBlur(GaussianBlur {
-                std_deviation: (std_dev, std_dev),
+                std_deviation: NumberOptionalNumber(std_dev, std_dev),
                 ..GaussianBlur::default()
             }),
         }
@@ -385,7 +385,7 @@ impl DropShadow {
             primitive: Primitive::default(),
             params: PrimitiveParams::GaussianBlur(GaussianBlur {
                 in1: Input::SourceAlpha,
-                std_deviation: (std_dev, std_dev),
+                std_deviation: NumberOptionalNumber(std_dev, std_dev),
                 ..GaussianBlur::default()
             }),
         }
@@ -608,19 +608,21 @@ impl Sepia {
 }
 
 impl Parse for FilterFunction {
+    #[allow(clippy::type_complexity)]
+    #[rustfmt::skip]
     fn parse<'i>(parser: &mut Parser<'i, '_>) -> Result<Self, crate::error::ParseError<'i>> {
         let loc = parser.current_source_location();
         let fns: Vec<(&str, &dyn Fn(&mut Parser<'i, '_>) -> _)> = vec![
-            ("blur", &parse_blur),
-            ("brightness", &parse_brightness),
-            ("contrast", &parse_contrast),
+            ("blur",        &parse_blur),
+            ("brightness",  &parse_brightness),
+            ("contrast",    &parse_contrast),
             ("drop-shadow", &parse_dropshadow),
-            ("grayscale", &parse_grayscale),
-            ("hue-rotate", &parse_huerotate),
-            ("invert", &parse_invert),
-            ("opacity", &parse_opacity),
-            ("saturate", &parse_saturate),
-            ("sepia", &parse_sepia),
+            ("grayscale",   &parse_grayscale),
+            ("hue-rotate",  &parse_huerotate),
+            ("invert",      &parse_invert),
+            ("opacity",     &parse_opacity),
+            ("saturate",    &parse_saturate),
+            ("sepia",       &parse_sepia),
         ];
 
         for (filter_name, parse_fn) in fns {
@@ -636,22 +638,18 @@ impl Parse for FilterFunction {
 impl FilterFunction {
     // If this function starts actually returning an Err, remove this Clippy exception:
     #[allow(clippy::unnecessary_wraps)]
-    pub fn to_filter_spec(
-        &self,
-        params: &NormalizeParams,
-        current_color: RGBA,
-    ) -> Result<FilterSpec, FilterResolveError> {
+    pub fn to_filter_spec(&self, params: &NormalizeParams, current_color: RGBA) -> FilterSpec {
         match self {
-            FilterFunction::Blur(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Brightness(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Contrast(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::DropShadow(v) => Ok(v.to_filter_spec(params, current_color)),
-            FilterFunction::Grayscale(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::HueRotate(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Invert(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Opacity(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Saturate(v) => Ok(v.to_filter_spec(params)),
-            FilterFunction::Sepia(v) => Ok(v.to_filter_spec(params)),
+            FilterFunction::Blur(v) => v.to_filter_spec(params),
+            FilterFunction::Brightness(v) => v.to_filter_spec(params),
+            FilterFunction::Contrast(v) => v.to_filter_spec(params),
+            FilterFunction::DropShadow(v) => v.to_filter_spec(params, current_color),
+            FilterFunction::Grayscale(v) => v.to_filter_spec(params),
+            FilterFunction::HueRotate(v) => v.to_filter_spec(params),
+            FilterFunction::Invert(v) => v.to_filter_spec(params),
+            FilterFunction::Opacity(v) => v.to_filter_spec(params),
+            FilterFunction::Saturate(v) => v.to_filter_spec(params),
+            FilterFunction::Sepia(v) => v.to_filter_spec(params),
         }
     }
 }
