@@ -8,14 +8,15 @@ use std::time::Instant;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::{Draw, ElementResult, SetAttributes};
-use crate::error::{ElementError, ParseError, RenderingError};
+use crate::element::{set_attribute, Draw, SetAttributes};
+use crate::error::{ParseError, RenderingError};
 use crate::filter::UserSpaceFilter;
 use crate::length::*;
-use crate::node::{Node, NodeBorrow};
+use crate::node::Node;
 use crate::paint_server::UserSpacePaintSource;
 use crate::parsers::{CustomIdent, Parse, ParseValue};
 use crate::properties::ColorInterpolationFilters;
+use crate::session::Session;
 use crate::surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 use crate::transform::Transform;
 use crate::xml::Attributes;
@@ -206,116 +207,49 @@ impl Primitive {
     fn parse_standard_attributes(
         &mut self,
         attrs: &Attributes,
-    ) -> Result<(Input, Input), ElementError> {
+        session: &Session,
+    ) -> (Input, Input) {
         let mut input_1 = Input::Unspecified;
         let mut input_2 = Input::Unspecified;
 
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => self.x = attr.parse(value)?,
-                expanded_name!("", "y") => self.y = attr.parse(value)?,
-                expanded_name!("", "width") => self.width = attr.parse(value)?,
-                expanded_name!("", "height") => self.height = attr.parse(value)?,
-                expanded_name!("", "result") => self.result = attr.parse(value)?,
-                expanded_name!("", "in") => input_1 = attr.parse(value)?,
-                expanded_name!("", "in2") => input_2 = attr.parse(value)?,
+                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
+                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
+                expanded_name!("", "width") => {
+                    set_attribute(&mut self.width, attr.parse(value), session)
+                }
+                expanded_name!("", "height") => {
+                    set_attribute(&mut self.height, attr.parse(value), session)
+                }
+                expanded_name!("", "result") => {
+                    set_attribute(&mut self.result, attr.parse(value), session)
+                }
+                expanded_name!("", "in") => set_attribute(&mut input_1, attr.parse(value), session),
+                expanded_name!("", "in2") => {
+                    set_attribute(&mut input_2, attr.parse(value), session)
+                }
                 _ => (),
             }
         }
 
-        Ok((input_1, input_2))
+        (input_1, input_2)
     }
 
-    pub fn parse_no_inputs(&mut self, attrs: &Attributes) -> ElementResult {
-        let (_, _) = self.parse_standard_attributes(attrs)?;
-        Ok(())
+    pub fn parse_no_inputs(&mut self, attrs: &Attributes, session: &Session) {
+        let (_, _) = self.parse_standard_attributes(attrs, session);
     }
 
-    pub fn parse_one_input(&mut self, attrs: &Attributes) -> Result<Input, ElementError> {
-        let (input_1, _) = self.parse_standard_attributes(attrs)?;
-        Ok(input_1)
+    pub fn parse_one_input(&mut self, attrs: &Attributes, session: &Session) -> Input {
+        let (input_1, _) = self.parse_standard_attributes(attrs, session);
+        input_1
     }
 
-    pub fn parse_two_inputs(&mut self, attrs: &Attributes) -> Result<(Input, Input), ElementError> {
-        self.parse_standard_attributes(attrs)
+    pub fn parse_two_inputs(&mut self, attrs: &Attributes, session: &Session) -> (Input, Input) {
+        self.parse_standard_attributes(attrs, session)
     }
 }
 
-pub fn extract_filter_from_filter_node(
-    filter_node: &Node,
-    acquired_nodes: &mut AcquiredNodes<'_>,
-    draw_ctx: &DrawingCtx,
-) -> Result<FilterSpec, FilterResolveError> {
-    let session = draw_ctx.session().clone();
-
-    let filter_node = &*filter_node;
-    assert!(is_element_of_type!(filter_node, Filter));
-
-    let filter_element = filter_node.borrow_element();
-
-    let user_space_filter = {
-        let filter_values = filter_element.get_computed_values();
-
-        let filter = borrow_element_as!(filter_node, Filter);
-
-        filter.to_user_space(&NormalizeParams::new(
-            filter_values,
-            &draw_ctx.get_view_params_for_units(filter.get_filter_units()),
-        ))
-    };
-
-    let primitives = filter_node
-        .children()
-        .filter(|c| c.is_element())
-        // Skip nodes in error.
-        .filter(|c| {
-            let in_error = c.borrow_element().is_in_error();
-
-            if in_error {
-                rsvg_log!(
-                    session,
-                    "(ignoring filter primitive {} because it is in error)",
-                    c
-                );
-            }
-
-            !in_error
-        })
-        // Keep only filter primitives (those that implement the Filter trait)
-        .filter(|c| c.borrow_element().as_filter_effect().is_some())
-        .map(|primitive_node| {
-            let elt = primitive_node.borrow_element();
-            let effect = elt.as_filter_effect().unwrap();
-
-            let primitive_name = format!("{}", primitive_node);
-
-            let primitive_values = elt.get_computed_values();
-            let params = NormalizeParams::new(
-                primitive_values,
-                &draw_ctx.get_view_params_for_units(user_space_filter.primitive_units),
-            );
-
-            effect
-                .resolve(acquired_nodes, &primitive_node)
-                .map_err(|e| {
-                    rsvg_log!(
-                        session,
-                        "(filter primitive {} returned an error: {})",
-                        primitive_name,
-                        e
-                    );
-                    e
-                })
-                .map(|primitive| primitive.into_user_space(&params))
-        })
-        .collect::<Result<Vec<UserSpacePrimitive>, FilterResolveError>>()?;
-
-    Ok(FilterSpec {
-        user_space_filter,
-        primitives,
-    })
-}
-
 /// Applies a filter and returns the resulting surface.
 pub fn render(
     filter: &FilterSpec,
