@@ -1,6 +1,6 @@
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::{ElementResult, SetAttributes};
+use crate::element::SetAttributes;
 use crate::node::Node;
 use crate::properties::ColorInterpolationFilters;
 use crate::rect::IRect;
@@ -28,9 +28,8 @@ pub struct Tile {
 }
 
 impl SetAttributes for FeTile {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
-        self.params.in1 = self.base.parse_one_input(attrs)?;
-        Ok(())
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
+        self.params.in1 = self.base.parse_one_input(attrs, session);
     }
 }
 
