@@ -6,7 +6,7 @@ use crate::aspect_ratio::AspectRatio;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::{Draw, ElementResult, SetAttributes};
+use crate::element::{set_attribute, Draw, SetAttributes};
 use crate::error::*;
 use crate::href::{is_href, set_href};
 use crate::layout::{self, StackingContext};
@@ -28,21 +28,21 @@ pub struct Image {
 }
 
 impl SetAttributes for Image {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "preserveAspectRatio") => self.aspect = attr.parse(value)?,
+                expanded_name!("", "preserveAspectRatio") => {
+                    set_attribute(&mut self.aspect, attr.parse(value), session)
+                }
 
                 // "path" is used by some older Adobe Illustrator versions
                 ref a if is_href(a) || *a == expanded_name!("", "path") => {
-                    set_href(a, &mut self.href, value.to_string())
+                    set_href(a, &mut self.href, Some(value.to_string()))
                 }
 
                 _ => (),
             }
         }
-
-        Ok(())
     }
 }
 
