@@ -3,6 +3,7 @@
 //! The idea is to take the DOM tree and produce a layout tree with SVG concepts.
 
 use std::rc::Rc;
+use std::sync::Arc;
 
 use crate::aspect_ratio::AspectRatio;
 use crate::bbox::BoundingBox;
@@ -18,7 +19,7 @@ use crate::properties::{
     ClipRule, ComputedValues, Direction, FillRule, Filter, FontFamily, FontStretch, FontStyle,
     FontVariant, FontWeight, Isolation, MixBlendMode, Opacity, Overflow, PaintOrder,
     ShapeRendering, StrokeDasharray, StrokeLinecap, StrokeLinejoin, StrokeMiterlimit,
-    TextDecoration, TextRendering, UnicodeBidi, XmlLang,
+    TextDecoration, TextRendering, UnicodeBidi, VectorEffect, XmlLang,
 };
 use crate::rect::Rect;
 use crate::session::Session;
@@ -62,20 +63,19 @@ pub struct Stroke {
     pub line_join: StrokeLinejoin,
     pub dash_offset: f64,
     pub dashes: Box<[f64]>,
+    // https://svgwg.org/svg2-draft/painting.html#non-scaling-stroke
+    pub non_scaling: bool,
 }
 
 /// Paths and basic shapes resolved to a path.
-///
-/// Note that `stroke_paint` and `fill_paint` are not in user-space coordinates;
-/// they are just resolved to a `PaintSource`.  Turning them to a `UserSpacePaintSource`
-/// involves knowing the bounding box of the path.
 pub struct Shape {
     pub path: Rc<Path>,
+    pub extents: Option<Rect>,
     pub is_visible: bool,
     pub paint_order: PaintOrder,
     pub stroke: Stroke,
-    pub stroke_paint: PaintSource,
-    pub fill_paint: PaintSource,
+    pub stroke_paint: UserSpacePaintSource,
+    pub fill_paint: UserSpacePaintSource,
     pub fill_rule: FillRule,
     pub clip_rule: ClipRule,
     pub shape_rendering: ShapeRendering,
@@ -86,8 +86,8 @@ pub struct Shape {
 
 pub struct Marker {
     pub node_ref: Option<Node>,
-    pub context_stroke: PaintSource,
-    pub context_fill: PaintSource,
+    pub context_stroke: Arc<PaintSource>,
+    pub context_fill: Arc<PaintSource>,
 }
 
 /// Image in user-space coordinates.
@@ -110,7 +110,6 @@ pub struct TextSpan {
     pub paint_order: PaintOrder,
     pub stroke: Stroke,
     pub stroke_paint: UserSpacePaintSource,
-    pub stroke_paint_source: PaintSource,
     pub fill_paint: UserSpacePaintSource,
     pub text_rendering: TextRendering,
     pub link_target: Option<String>,
@@ -251,6 +250,7 @@ impl Stroke {
         let line_cap = values.stroke_line_cap();
         let line_join = values.stroke_line_join();
         let dash_offset = values.stroke_dashoffset().0.to_user(params);
+        let non_scaling = values.vector_effect() == VectorEffect::NonScalingStroke;
 
         let dashes = match values.stroke_dasharray() {
             StrokeDasharray(Dasharray::None) => Box::new([]),
@@ -267,6 +267,7 @@ impl Stroke {
             line_join,
             dash_offset,
             dashes,
+            non_scaling,
         }
     }
 }
