@@ -44,12 +44,7 @@
 //!     let renderer = librsvg::CairoRenderer::new(&handle);
 //!     renderer.render_document(
 //!         &cr,
-//!         &cairo::Rectangle {
-//!             x: 0.0,
-//!             y: 0.0,
-//!             width: f64::from(WIDTH),
-//!             height: f64::from(HEIGHT),
-//!         },
+//!         &cairo::Rectangle::new(0.0, 0.0, f64::from(WIDTH), f64::from(HEIGHT))
 //!     ).unwrap();
 //!
 //!     // Loading from a static SVG asset
@@ -105,30 +100,28 @@
 //! include raster image data, encoded as base-64, directly in an SVG
 //! file.
 //!
-//! 2. URLs with queries ("?") or fragment identifiers ("#") are not allowed.
-//!
-//! 3. All URL schemes other than data: in references require a base URL.  For
+//! 2. All other URL schemes in references require a base URL.  For
 //! example, this means that if you load an SVG with [`Loader::read_stream`]
 //! without providing a `base_file`, then any referenced files will not
 //! be allowed (e.g. raster images to be loaded from other files will
 //! not work).
 //!
-//! 4. If referenced URLs are absolute, rather than relative, then
+//! 3. If referenced URLs are absolute, rather than relative, then
 //! they must have the same scheme as the base URL.  For example, if
 //! the base URL has a "`file`" scheme, then all URL references inside
 //! the SVG must also have the "`file`" scheme, or be relative
 //! references which will be resolved against the base URL.
 //!
-//! 5. If referenced URLs have a "`resource`" scheme, that is, if they
+//! 4. If referenced URLs have a "`resource`" scheme, that is, if they
 //! are included into your binary program with GLib's resource
 //! mechanism, they are allowed to be loaded (provided that the base
 //! URL is also a "`resource`", per the previous rule).
 //!
-//! 6. Otherwise, non-`file` schemes are not allowed.  For example,
+//! 5. Otherwise, non-`file` schemes are not allowed.  For example,
 //! librsvg will not load `http` resources, to keep malicious SVG data
 //! from "phoning home".
 //!
-//! 7. A relative URL must resolve to the same directory as the base
+//! 6. A relative URL must resolve to the same directory as the base
 //! URL, or to one of its subdirectories.  Librsvg will canonicalize
 //! filenames, by removing "`..`" path components and resolving symbolic
 //! links, to decide whether files meet these conditions.
@@ -138,10 +131,12 @@
 //! [SVG 1.1]: https://www.w3.org/TR/SVG11/
 //! [SVG 2]: https://www.w3.org/TR/SVG2/
 
+#![doc(html_logo_url = "https://gnome.pages.gitlab.gnome.org/librsvg/Rsvg-2.0/librsvg-r.svg")]
 #![allow(rustdoc::private_intra_doc_links)]
 #![allow(clippy::clone_on_ref_ptr)]
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
 #![allow(clippy::too_many_arguments)]
+#![allow(clippy::derive_partial_eq_without_eq)]
 #![warn(nonstandard_style, rust_2018_idioms, unused)]
 // Some lints no longer exist
 #![warn(renamed_and_removed_lints)]
@@ -250,7 +245,8 @@ pub mod rsvg_convert_only {
     pub use crate::dpi::Dpi;
     pub use crate::error::ParseError;
     pub use crate::length::{
-        CssLength, Horizontal, Length, Normalize, NormalizeParams, ULength, Validate, Vertical,
+        CssLength, Horizontal, Length, Normalize, NormalizeParams, Signed, ULength, Unsigned,
+        Validate, Vertical,
     };
     pub use crate::parsers::{Parse, ParseValue};
     pub use crate::rect::Rect;
