@@ -9,7 +9,7 @@ use std::rc::Rc;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::{Draw, ElementResult, SetAttributes};
+use crate::element::{set_attribute, Draw, SetAttributes};
 use crate::error::*;
 use crate::iri::Iri;
 use crate::layout::{Marker, Shape, StackingContext, Stroke};
@@ -42,6 +42,113 @@ trait BasicShape {
     fn make_shape(&self, params: &NormalizeParams, values: &ComputedValues) -> ShapeDef;
 }
 
+fn draw_basic_shape(
+    basic_shape: &dyn BasicShape,
+    node: &Node,
+    acquired_nodes: &mut AcquiredNodes<'_>,
+    cascaded: &CascadedValues<'_>,
+    draw_ctx: &mut DrawingCtx,
+    clipping: bool,
+) -> Result<BoundingBox, RenderingError> {
+    let values = cascaded.get();
+    let view_params = draw_ctx.get_view_params();
+    let params = NormalizeParams::new(values, &view_params);
+    let shape_def = basic_shape.make_shape(&params, values);
+
+    let is_visible = values.is_visible();
+    let paint_order = values.paint_order();
+
+    let stroke = Stroke::new(values, &params);
+
+    let session = draw_ctx.session();
+
+    let stroke_paint = values.stroke().0.resolve(
+        acquired_nodes,
+        values.stroke_opacity().0,
+        values.color().0,
+        cascaded.context_fill.clone(),
+        cascaded.context_stroke.clone(),
+        session,
+    );
+
+    let fill_paint = values.fill().0.resolve(
+        acquired_nodes,
+        values.fill_opacity().0,
+        values.color().0,
+        cascaded.context_fill.clone(),
+        cascaded.context_stroke.clone(),
+        session,
+    );
+
+    let fill_rule = values.fill_rule();
+    let clip_rule = values.clip_rule();
+    let shape_rendering = values.shape_rendering();
+
+    let marker_start_node;
+    let marker_mid_node;
+    let marker_end_node;
+
+    if shape_def.markers == Markers::Yes {
+        marker_start_node = acquire_marker(session, acquired_nodes, &values.marker_start().0);
+        marker_mid_node = acquire_marker(session, acquired_nodes, &values.marker_mid().0);
+        marker_end_node = acquire_marker(session, acquired_nodes, &values.marker_end().0);
+    } else {
+        marker_start_node = None;
+        marker_mid_node = None;
+        marker_end_node = None;
+    }
+
+    let marker_start = Marker {
+        node_ref: marker_start_node,
+        context_stroke: stroke_paint.clone(),
+        context_fill: fill_paint.clone(),
+    };
+
+    let marker_mid = Marker {
+        node_ref: marker_mid_node,
+        context_stroke: stroke_paint.clone(),
+        context_fill: fill_paint.clone(),
+    };
+
+    let marker_end = Marker {
+        node_ref: marker_end_node,
+        context_stroke: stroke_paint.clone(),
+        context_fill: fill_paint.clone(),
+    };
+
+    let extents = draw_ctx.compute_path_extents(&shape_def.path)?;
+
+    let stroke_paint = stroke_paint.to_user_space(&extents, &view_params, values);
+    let fill_paint = fill_paint.to_user_space(&extents, &view_params, values);
+
+    let shape = Shape {
+        path: shape_def.path,
+        extents,
+        is_visible,
+        paint_order,
+        stroke,
+        stroke_paint,
+        fill_paint,
+        fill_rule,
+        clip_rule,
+        shape_rendering,
+        marker_start,
+        marker_mid,
+        marker_end,
+    };
+
+    let elt = node.borrow_element();
+    let stacking_ctx = StackingContext::new(
+        draw_ctx.session(),
+        acquired_nodes,
+        &elt,
+        values.transform(),
+        values,
+    );
+
+    draw_ctx.draw_shape(&shape, &stacking_ctx, acquired_nodes, values, clipping)
+}
+
 macro_rules! impl_draw {
     ($name:ident) => {
         impl Draw for $name {
@@ -53,107 +160,7 @@ macro_rules! impl_draw {
                 draw_ctx: &mut DrawingCtx,
                 clipping: bool,
             ) -> Result<BoundingBox, RenderingError> {
-                let values = cascaded.get();
-                let view_params = draw_ctx.get_view_params();
-                let params = NormalizeParams::new(values, &view_params);
-                let shape_def = self.make_shape(&params, values);
-
-                let is_visible = values.is_visible();
-                let paint_order = values.paint_order();
-
-                let stroke = Stroke::new(values, &params);
-
-                let session = draw_ctx.session();
-
-                let stroke_paint = values.stroke().0.resolve(
-                    acquired_nodes,
-                    values.stroke_opacity().0,
-                    values.color().0,
-                    cascaded.context_fill.clone(),
-                    cascaded.context_stroke.clone(),
-                    session,
-                );
-
-                let fill_paint = values.fill().0.resolve(
-                    acquired_nodes,
-                    values.fill_opacity().0,
-                    values.color().0,
-                    cascaded.context_fill.clone(),
-                    cascaded.context_stroke.clone(),
-                    session,
-                );
-
-                let fill_rule = values.fill_rule();
-                let clip_rule = values.clip_rule();
-                let shape_rendering = values.shape_rendering();
-
-                let marker_start_node;
-                let marker_mid_node;
-                let marker_end_node;
-
-                if shape_def.markers == Markers::Yes {
-                    marker_start_node =
-                        acquire_marker(session, acquired_nodes, &values.marker_start().0);
-                    marker_mid_node =
-                        acquire_marker(session, acquired_nodes, &values.marker_mid().0);
-                    marker_end_node =
-                        acquire_marker(session, acquired_nodes, &values.marker_end().0);
-                } else {
-                    marker_start_node = None;
-                    marker_mid_node = None;
-                    marker_end_node = None;
-                }
-
-                let marker_start = Marker {
-                    node_ref: marker_start_node,
-                    context_stroke: stroke_paint.clone(),
-                    context_fill: fill_paint.clone(),
-                };
-
-                let marker_mid = Marker {
-                    node_ref: marker_mid_node,
-                    context_stroke: stroke_paint.clone(),
-                    context_fill: fill_paint.clone(),
-                };
-
-                let marker_end = Marker {
-                    node_ref: marker_end_node,
-                    context_stroke: stroke_paint.clone(),
-                    context_fill: fill_paint.clone(),
-                };
-
-                let shape = Shape {
-                    path: shape_def.path,
-                    is_visible,
-                    paint_order,
-                    stroke,
-                    stroke_paint,
-                    fill_paint,
-                    fill_rule,
-                    clip_rule,
-                    shape_rendering,
-                    marker_start,
-                    marker_mid,
-                    marker_end,
-                };
-
-                let elt = node.borrow_element();
-                let stacking_ctx = StackingContext::new(
-                    draw_ctx.session(),
-                    acquired_nodes,
-                    &elt,
-                    values.transform(),
-                    values,
-                );
-
-                draw_ctx.draw_shape(
-                    &view_params,
-                    &shape,
-                    &stacking_ctx,
-                    acquired_nodes,
-                    values,
-                    clipping,
-                )
+                draw_basic_shape(self, node, acquired_nodes, cascaded, draw_ctx, clipping)
             }
         }
     };
@@ -248,21 +255,19 @@ pub struct Path {
 impl_draw!(Path);
 
 impl SetAttributes for Path {
-    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "d") {
                 let mut builder = PathBuilder::default();
                 if let Err(e) = builder.parse(value) {
-                    // FIXME: we don't propagate errors upstream, but creating a partial
-                    // path is OK per the spec
+                    // Creating a partial path is OK per the spec; we don't throw away the partial
+                    // result in case of an error.
 
                     rsvg_log!(session, "could not parse path: {}", e);
                 }
                 self.path = Rc::new(builder.into_path());
             }
         }
-
-        Ok(())
     }
 }
 
@@ -339,14 +344,12 @@ pub struct Polygon {
 impl_draw!(Polygon);
 
 impl SetAttributes for Polygon {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "points") {
-                self.points = attr.parse(value)?;
+                set_attribute(&mut self.points, attr.parse(value), session);
             }
         }
-
-        Ok(())
     }
 }
 
@@ -364,14 +367,12 @@ pub struct Polyline {
 impl_draw!(Polyline);
 
 impl SetAttributes for Polyline {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "points") {
-                self.points = attr.parse(value)?;
+                set_attribute(&mut self.points, attr.parse(value), session);
             }
         }
-
-        Ok(())
     }
 }
 
@@ -392,18 +393,16 @@ pub struct Line {
 impl_draw!(Line);
 
 impl SetAttributes for Line {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x1") => self.x1 = attr.parse(value)?,
-                expanded_name!("", "y1") => self.y1 = attr.parse(value)?,
-                expanded_name!("", "x2") => self.x2 = attr.parse(value)?,
-                expanded_name!("", "y2") => self.y2 = attr.parse(value)?,
+                expanded_name!("", "x1") => set_attribute(&mut self.x1, attr.parse(value), session),
+                expanded_name!("", "y1") => set_attribute(&mut self.y1, attr.parse(value), session),
+                expanded_name!("", "x2") => set_attribute(&mut self.x2, attr.parse(value), session),
+                expanded_name!("", "y2") => set_attribute(&mut self.y2, attr.parse(value), session),
                 _ => (),
             }
         }
-
-        Ok(())
     }
 }
 
