@@ -5,11 +5,12 @@ use pango::IsAttribute;
 use std::cell::RefCell;
 use std::convert::TryFrom;
 use std::rc::Rc;
+use std::sync::Arc;
 
 use crate::bbox::BoundingBox;
 use crate::document::{AcquiredNodes, NodeId};
 use crate::drawing_ctx::{create_pango_context, DrawingCtx, FontOptions, ViewParams};
-use crate::element::{Draw, Element, ElementResult, SetAttributes};
+use crate::element::{set_attribute, Draw, Element, SetAttributes};
 use crate::error::*;
 use crate::layout::{self, FontProperties, StackingContext, Stroke, TextSpan};
 use crate::length::*;
@@ -120,8 +121,8 @@ struct LayoutSpan {
     y: f64,
     paint_order: PaintOrder,
     stroke: Stroke,
-    stroke_paint: PaintSource,
-    fill_paint: PaintSource,
+    stroke_paint: Arc<PaintSource>,
+    fill_paint: Arc<PaintSource>,
     text_rendering: TextRendering,
     link_target: Option<String>,
     values: Rc<ComputedValues>,
@@ -445,7 +446,7 @@ impl PositionedSpan {
 
         let stroke = Stroke::new(&self.values, &params);
 
-        let gravity = layout.context().unwrap().gravity();
+        let gravity = layout.context().gravity();
 
         let bbox = compute_text_box(&layout, x, y, layout_context.transform, gravity);
 
@@ -654,7 +655,7 @@ impl Chars {
         depth: usize,
         link_target: Option<String>,
     ) -> Span {
-        self.ensure_normalized_string(node, &*values);
+        self.ensure_normalized_string(node, &values);
 
         Span::new(
             self.space_normalized.borrow().as_ref().unwrap(),
@@ -732,18 +733,16 @@ impl Text {
 }
 
 impl SetAttributes for Text {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => self.x = attr.parse(value)?,
-                expanded_name!("", "y") => self.y = attr.parse(value)?,
-                expanded_name!("", "dx") => self.dx = attr.parse(value)?,
-                expanded_name!("", "dy") => self.dy = attr.parse(value)?,
+                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
+                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
+                expanded_name!("", "dx") => set_attribute(&mut self.dx, attr.parse(value), session),
+                expanded_name!("", "dy") => set_attribute(&mut self.dy, attr.parse(value), session),
                 _ => (),
             }
         }
-
-        Ok(())
     }
 }
 
@@ -829,12 +828,12 @@ impl Draw for Text {
                 let mut text_spans = Vec::new();
                 for span in layout_spans {
                     let stroke_paint = span.stroke_paint.to_user_space(
-                        &text_bbox,
+                        &text_bbox.rect,
                         &layout_context.view_params,
                         &span.values,
                     );
                     let fill_paint = span.fill_paint.to_user_space(
-                        &text_bbox,
+                        &text_bbox.rect,
                         &layout_context.view_params,
                         &span.values,
                     );
@@ -849,7 +848,6 @@ impl Draw for Text {
                         paint_order: span.paint_order,
                         stroke: span.stroke,
                         stroke_paint,
-                        stroke_paint_source: span.stroke_paint,
                         fill_paint,
                         text_rendering: span.text_rendering,
                         link_target: span.link_target,
@@ -926,7 +924,7 @@ fn extract_chars_children_to_chunks_recursively(
 }
 
 impl SetAttributes for TRef {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) {
         self.link = attrs
             .iter()
             .find(|(attr, _)| attr.expanded() == expanded_name!(xlink "href"))
@@ -934,8 +932,6 @@ impl SetAttributes for TRef {
             // the <tref> element got removed in SVG2.  So, here we still use a match
             // against the full namespaced version of the attribute.
             .and_then(|(attr, value)| NodeId::parse(value).attribute(attr).ok());
-
-        Ok(())
     }
 }
 
@@ -994,18 +990,16 @@ impl TSpan {
 }
 
 impl SetAttributes for TSpan {
-    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => self.x = attr.parse(value)?,
-                expanded_name!("", "y") => self.y = attr.parse(value)?,
-                expanded_name!("", "dx") => self.dx = attr.parse(value)?,
-                expanded_name!("", "dy") => self.dy = attr.parse(value)?,
+                expanded_name!("", "x") => set_attribute(&mut self.x, attr.parse(value), session),
+                expanded_name!("", "y") => set_attribute(&mut self.y, attr.parse(value), session),
+                expanded_name!("", "dx") => set_attribute(&mut self.dx, attr.parse(value), session),
+                expanded_name!("", "dy") => set_attribute(&mut self.dy, attr.parse(value), session),
                 _ => (),
             }
         }
-
-        Ok(())
     }
 }
 
@@ -1206,7 +1200,7 @@ fn create_pango_layout(
         create_pango_context(&layout_context.font_options, &layout_context.transform);
 
     if let XmlLang(Some(ref lang)) = props.xml_lang {
-        pango_context.set_language(&pango::Language::from_string(lang.as_str()));
+        pango_context.set_language(Some(&pango::Language::from_string(lang.as_str())));
     }
 
     pango_context.set_base_gravity(pango::Gravity::from(layout_context.writing_mode));
