@@ -32,20 +32,14 @@ fn rendering_instancing_limit(name: &str) {
         .unwrap_or_else(|e| panic!("could not load: {}", e));
 
     let surface = cairo::ImageSurface::create(cairo::Format::ARgb32, 500, 500).unwrap();
-    let cr = cairo::Context::new(&surface);
+    let cr = cairo::Context::new(&surface).expect("Failed to create a cairo context");
 
     // Note that at least 515-patttern-billion-laughs.svg requires a viewport of this size
     // or bigger; a smaller one causes the recursive patterns to get so small that they
     // are culled out, and so the document doesn't reach the instancing limit.
-    match CairoRenderer::new(&handle).render_document(
-        &cr,
-        &cairo::Rectangle {
-            x: 0.0,
-            y: 0.0,
-            width: 500.0,
-            height: 500.0,
-        },
-    ) {
+    match CairoRenderer::new(&handle)
+        .render_document(&cr, &cairo::Rectangle::new(0.0, 0.0, 500.0, 500.0))
+    {
         Ok(_) => (),
         Err(RenderingError::LimitExceeded(ImplementationLimit::TooManyReferencedElements)) => (),
         _ => panic!("unexpected error code"),
