@@ -71,12 +71,7 @@ fn reference_test(path: &str) {
         renderer
             .render_document(
                 &cr,
-                &cairo::Rectangle {
-                    x: 0.0,
-                    y: 0.0,
-                    width: f64::from(width),
-                    height: f64::from(height),
-                },
+                &cairo::Rectangle::new(0.0, 0.0, f64::from(width), f64::from(height)),
             )
             .unwrap();
     }
@@ -141,7 +136,7 @@ fn image_size(dim: IntrinsicDimensions, dpi: f64) -> (i32, i32) {
         (Length { length: w, unit: Percent },
          Length { length: h, unit: Percent }, vbox) if w == 1.0 && h == 1.0 => {
             if let Some(vbox) = vbox {
-                (vbox.width, vbox.height)
+                (vbox.width(), vbox.height())
             } else {
                 panic!("SVG with percentage width/height must have a viewBox");
             }
@@ -401,6 +396,12 @@ test_svg_reference!(
     "tests/fixtures/reftests/svg2-reftests/mask-and-opacity-ref.svg"
 );
 
+test_svg_reference!(
+    gaussian_blur_nonpositive_913,
+    "tests/fixtures/reftests/svg2-reftests/913-gaussian-blur-nonpositive.svg",
+    "tests/fixtures/reftests/svg2-reftests/913-gaussian-blur-nonpositive-ref.svg"
+);
+
 test_svg_reference!(
     bug_880_horizontal_vertical_stroked_lines,
     "tests/fixtures/reftests/bugs-reftests/880-stroke-wide-line.svg",
@@ -412,3 +413,15 @@ test_svg_reference!(
     "tests/fixtures/reftests/bugs-reftests/92-symbol-clip.svg",
     "tests/fixtures/reftests/bugs-reftests/92-symbol-clip-ref.svg"
 );
+
+test_svg_reference!(
+    bug_875_svg_use_width_height,
+    "tests/fixtures/reftests/bugs-reftests/875-svg-use-width-height.svg",
+    "tests/fixtures/reftests/bugs-reftests/875-svg-use-width-height-ref.svg"
+);
+
+test_svg_reference!(
+    bug_885_vector_effect_non_scaling_stroke,
+    "tests/fixtures/reftests/bugs-reftests/885-vector-effect-non-scaling-stroke.svg",
+    "tests/fixtures/reftests/bugs-reftests/885-vector-effect-non-scaling-stroke-ref.svg"
+);
